@@ -1,10 +1,18 @@
 import base64
 import json
 import os
-import requests
-import pandas as pd
 from importlib.metadata import version
-from defog import generate_schema, query_methods, admin_methods
+from defog import (
+    generate_schema,
+    async_generate_schema,
+    query_methods,
+    async_query_methods,
+    admin_methods,
+    async_admin_methods,
+    health_methods,
+    async_health_methods,
+)
+from typing import Dict, List, Optional, Union, Any
 
 try:
     __version__ = version("defog")
@@ -18,14 +26,21 @@ SUPPORTED_DB_TYPES = [
     "bigquery",
     "snowflake",
     "databricks",
+    "sqlserver",
 ]
 
 
-class Defog:
+class BaseDefog:
     """
-    The main class for Defog
+    The base class for Defog and AsyncDefog
     """
 
+    api_key: str
+    db_type: str
+    db_creds: dict
+    base_url: str
+    generate_query_url: str
+
     def __init__(
         self,
         api_key: str = "",
@@ -35,9 +50,10 @@ class Defog:
         save_json: bool = True,
         base_url: str = "https://api.defog.ai",
         generate_query_url: str = "https://api.defog.ai/generate_query_chat",
+        verbose: bool = False,
     ):
         """
-        Initializes the Defog class.
+        Initializes the Base Defog class.
         We have the possible scenarios detailed below:
         1) no config file, no/incomplete params -> success if only db_creds missing, error otherwise
         2) no config file, wrong params -> error
@@ -64,7 +80,10 @@ class Defog:
                 self.save_connection_json()
         elif os.path.exists(self.filepath):  # case 4 and 5
             # read connection details from filepath
-            print("Connection details found. Reading connection details from file...")
+            if verbose:
+                print(
+                    "Connection details found. Reading connection details from file..."
+                )
             if api_key == "":
                 with open(self.filepath, "r") as f:
                     data = json.load(f)
@@ -76,9 +95,10 @@ class Defog:
                         self.base_url = data.get("base_url", "https://api.defog.ai")
                         self.generate_query_url = data.get(
                             "generate_query_url",
-                            "https://api.defog.ai/generate_query_chat",
+                            f"{self.base_url}/generate_query_chat",
                         )
-                        print(f"Connection details read from {self.filepath}.")
+                        if verbose:
+                            print(f"Connection details read from {self.filepath}.")
                     else:
                         raise KeyError(
                             f"Invalid file at {self.filepath}.\n"
@@ -160,7 +180,14 @@ class Defog:
                 raise KeyError("db_creds must contain a 'access_token' key.")
             if "http_path" not in db_creds:
                 raise KeyError("db_creds must contain a 'http_path' key.")
-        elif db_type == "mongo" or db_type == "sqlserver":
+        elif db_type == "sqlserver":
+            if "server" not in db_creds:
+                raise KeyError("db_creds must contain a 'server' key.")
+            if "user" not in db_creds:
+                raise KeyError("db_creds must contain a 'user' key.")
+            if "password" not in db_creds:
+                raise KeyError("db_creds must contain a 'password' key.")
+        elif db_type == "mongo":
             if "connection_string" not in db_creds:
                 raise KeyError("db_creds must contain a 'connection_string' key.")
         elif db_type == "bigquery":
@@ -191,6 +218,128 @@ class Defog:
         self.db_creds = creds["db_creds"]
 
 
+class Defog(BaseDefog):
+    """
+    The main class for Defog (Synchronous)
+    """
+
+    def __init__(
+        self,
+        api_key: str = "",
+        db_type: str = "",
+        db_creds: dict = {},
+        base64creds: str = "",
+        save_json: bool = True,
+        base_url: str = "https://api.defog.ai",
+        generate_query_url: str = "https://api.defog.ai/generate_query_chat",
+        verbose: bool = False,
+    ):
+        """Initializes the synchronous version of the Defog class"""
+        super().__init__(
+            api_key=api_key,
+            db_type=db_type,
+            db_creds=db_creds,
+            base64creds=base64creds,
+            save_json=save_json,
+            base_url=base_url,
+            generate_query_url=generate_query_url,
+            verbose=verbose,
+        )
+
+    def generate_db_schema(
+        self,
+        tables: list,
+        scan: bool = True,
+        upload: bool = True,
+        return_tables_only: bool = False,
+        return_format: str = "csv",
+    ) -> str: ...
+
+    def run_query(
+        self,
+        question: str,
+        hard_filters: str = "",
+        previous_context: list = [],
+        glossary: str = "",
+        query: dict = None,
+        retries: int = 3,
+        dev: bool = False,
+        temp: bool = False,
+        profile: bool = False,
+        ignore_cache: bool = False,
+        model: str = "",
+        use_golden_queries: bool = True,
+        subtable_pruning: bool = False,
+        glossary_pruning: bool = False,
+        prune_max_tokens: int = 2000,
+        prune_bm25_num_columns: int = 10,
+        prune_glossary_max_tokens: int = 1000,
+        prune_glossary_num_cos_sim_units: int = 10,
+        prune_glossary_bm25_units: int = 10,
+    ): ...
+
+
+class AsyncDefog(BaseDefog):
+    """
+    The main class for Defog (Asynchronous)
+    """
+
+    def __init__(
+        self,
+        api_key: str = "",
+        db_type: str = "",
+        db_creds: dict = {},
+        base64creds: str = "",
+        save_json: bool = True,
+        base_url: str = "https://api.defog.ai",
+        generate_query_url: str = "https://api.defog.ai/generate_query_chat",
+        verbose: bool = False,
+    ):
+        """Initializes the asynchronous version of the Defog class"""
+        super().__init__(
+            api_key=api_key,
+            db_type=db_type,
+            db_creds=db_creds,
+            base64creds=base64creds,
+            save_json=save_json,
+            base_url=base_url,
+            generate_query_url=generate_query_url,
+            verbose=verbose,
+        )
+
+    async def generate_db_schema(
+        self,
+        tables: list,
+        scan: bool = True,
+        upload: bool = True,
+        return_tables_only: bool = False,
+        return_format: str = "csv",
+    ) -> str: ...
+
+    async def run_query(
+        self,
+        question: str,
+        hard_filters: str = "",
+        previous_context: list = [],
+        glossary: str = "",
+        query: dict = None,
+        retries: int = 3,
+        dev: bool = False,
+        temp: bool = False,
+        profile: bool = False,
+        ignore_cache: bool = False,
+        model: str = "",
+        use_golden_queries: bool = True,
+        subtable_pruning: bool = False,
+        glossary_pruning: bool = False,
+        prune_max_tokens: int = 2000,
+        prune_bm25_num_columns: int = 10,
+        prune_glossary_max_tokens: int = 1000,
+        prune_glossary_num_cos_sim_units: int = 10,
+        prune_glossary_bm25_units: int = 10,
+    ): ...
+
+
 # Add all methods from generate_schema to Defog
 for name in dir(generate_schema):
     attr = getattr(generate_schema, name)
@@ -198,6 +347,13 @@ for name in dir(generate_schema):
         # Add the method to Defog
         setattr(Defog, name, attr)
 
+# Add all methods from async_generate_schema to AsyncDefog
+for name in dir(async_generate_schema):
+    attr = getattr(async_generate_schema, name)
+    if callable(attr):
+        # Add the method to AsyncDefog
+        setattr(AsyncDefog, name, attr)
+
 # Add all methods from query_methods to Defog
 for name in dir(query_methods):
     attr = getattr(query_methods, name)
@@ -205,9 +361,37 @@ for name in dir(query_methods):
         # Add the method to Defog
         setattr(Defog, name, attr)
 
+# Add all methods from async_query_methods to AsyncDefog
+for name in dir(async_query_methods):
+    attr = getattr(async_query_methods, name)
+    if callable(attr):
+        # Add the method to AsyncDefog
+        setattr(AsyncDefog, name, attr)
+
 # Add all methods from admin_methods to Defog
 for name in dir(admin_methods):
     attr = getattr(admin_methods, name)
     if callable(attr):
         # Add the method to Defog
         setattr(Defog, name, attr)
+
+# Add all methods from async_admin_methods to AsyncDefog
+for name in dir(async_admin_methods):
+    attr = getattr(async_admin_methods, name)
+    if callable(attr):
+        # Add the method to AsyncDefog
+        setattr(AsyncDefog, name, attr)
+
+# Add all methods from health_methods to Defog
+for name in dir(health_methods):
+    attr = getattr(health_methods, name)
+    if callable(attr):
+        # Add the method to Defog
+        setattr(Defog, name, attr)
+
+# Add all methods from async_health_methods to AsyncDefog
+for name in dir(async_health_methods):
+    attr = getattr(async_health_methods, name)
+    if callable(attr):
+        # Add the method to AsyncDefog
+        setattr(AsyncDefog, name, attr)
