@@ -1,18 +1,20 @@
 import datetime
 import decimal
-import pwinput
 import json
 import os
 import re
 import shutil
 import subprocess
 import sys
-import requests
 import time
 
+import pandas as pd
+import pwinput
+import requests
+from prompt_toolkit import prompt
+
 import defog
 from defog.util import get_feedback, parse_update
-from prompt_toolkit import prompt
 
 USAGE_STRING = """
 Usage: defog <command>
@@ -39,10 +41,16 @@ def main():
         init()
     if sys.argv[1] == "gen":
         gen()
+    elif sys.argv[1] == "vet" and sys.argv[2] == "metadata":
+        vet_metadata()
     elif sys.argv[1] == "update":
         update()
+    elif sys.argv[1] == "glossary":
+        glossary()
     elif sys.argv[1] == "query":
         query()
+    elif sys.argv[1] == "golden":
+        golden()
     elif sys.argv[1] == "deploy":
         deploy()
     elif sys.argv[1] == "quota":
@@ -186,6 +194,17 @@ def init():
         db_creds = {
             "json_key_path": json_key_path,
         }
+    elif db_type == "sqlserver":
+        server = prompt("Please enter your database server host\n").strip()
+        database = prompt("Please enter your database name\n").strip()
+        user = prompt("Please enter your database user\n").strip()
+        password = pwinput.pwinput("Please enter your database password\n")
+        db_creds = {
+            "server": server,
+            "database": database,
+            "user": user,
+            "password": password,
+        }
 
     # write to filepath and print confirmation
     with open(filepath, "w") as f:
@@ -312,6 +331,73 @@ def gen():
     )
 
 
+def vet_metadata():
+    """
+    This function reads the metadata provided by the user, and for each
+    row, if it's column description is > 100 characters, it will
+    call /shorten_column_description to propose a shorter description.
+    Finally, it prompts the user to confirm and save the changes.
+    """
+    # get csv_path from sys.argv or prompt
+    if len(sys.argv) < 4:
+        print(
+            "defog vet-metadata requires a path to a CSV that contains your Database metadata. Please enter the path to the CSV you would like to vet:"
+        )
+        csv_path = prompt().strip()
+    else:
+        csv_path = sys.argv[3]
+    while not os.path.exists(csv_path):
+        print(f"File {csv_path} not found. Please enter a valid path:")
+        csv_path = prompt().strip()
+
+    dfg = defog.Defog()  # load config from .defog/connection.json
+    schema_df = pd.read_csv(csv_path).fillna("")
+    changes_made = False
+    for i, row in schema_df.iterrows():
+        column_name = row["column_name"]
+        column_description = row["column_description"]
+        if len(column_description) > 100:
+            print(
+                f"Column description for \033[1m{column_name}\033[0m is > 100 characters.\n"
+            )
+            print(f"Old description: {column_description}\n")
+            r = requests.post(
+                f"{dfg.base_url}/shorten_column_description",
+                json={
+                    "api_key": dfg.api_key,
+                    "column_name": column_name,
+                    "column_description": column_description,
+                },
+                verify=False,
+            )
+            resp = r.json()
+            new_column_description = resp["column_description"]
+            # show both descriptions and prompt user if they want to save
+            print(f"Suggested description: {new_column_description}\n")
+            reply = prompt(
+                "Do you want to save this description? Enter (Y/n) or your proposed edit: "
+            )
+            if reply.lower() == "y":
+                schema_df.at[i, "column_description"] = new_column_description
+                changes_made = True
+            elif reply.lower() == "n":
+                pass
+            else:
+                schema_df.at[i, "column_description"] = reply
+                changes_made = True
+    if not changes_made:
+        print("Your metadata is fine! No changes were made.")
+        sys.exit(0)
+    if_save = prompt(
+        f"Would you like to save the changes back to {csv_path}? Enter 'y' to save or anything else to exit without saving:"
+    )
+    if if_save.lower() == "y":
+        schema_df.to_csv(csv_path, index=False)
+        print(f"Changes saved to {csv_path}")
+    else:
+        print("Changes not saved.")
+
+
 def update():
     """
     Update the schema in defog with a new schema using the url provided.
@@ -326,6 +412,7 @@ def update():
         filename = sys.argv[2]
     # load config from .defog/connection.json
     df = defog.Defog()
+    # TODO call vet_schema
     # upload schema to defog
     resp = df.update_db_schema(filename)
     if resp["status"] == "success":
@@ -336,6 +423,60 @@ def update():
         print("Please try again, or contact us at founders@defog.ai")
 
 
+def glossary():
+    """
+    Checks for the next action in sys.argv and runs the appropriate function.
+    Available actions are: update, get, delete
+    If the sys.argv is missing an action, prompt the user for the action.
+    """
+    if len(sys.argv) < 3:
+        print(
+            "defog glossary requires an action. Please enter 'update', 'get', or 'delete':"
+        )
+        action = prompt().strip().lower()
+    else:
+        action = sys.argv[2].lower()
+    while action not in ["update", "get", "delete", "exit"]:
+        print("Please enter 'update', 'get', 'delete', or 'exit' to exit:")
+        action = prompt().strip().lower()
+
+    dfg = defog.Defog()
+    if action == "update":
+        # get path from sys.argv or prompt
+        if len(sys.argv) < 4:
+            print(
+                "defog glossary update requires a path to a .txt file containing your glossary:"
+            )
+            path = prompt().strip()
+        else:
+            path = sys.argv[3]
+        while not os.path.exists(path):
+            print(f"File {path} not found. Please enter a valid path:")
+            path = prompt().strip()
+        with open(path, "r") as f:
+            glossary = f.read()
+        resp = dfg.update_glossary(glossary)
+        if resp["status"] == "success":
+            print("Your glossary has been updated.")
+    elif action == "get":
+        glossary = dfg.get_glossary()
+        if not glossary:
+            print("You don't have a glossary yet.")
+            return
+        print(f"Your glossary is:\n{glossary}")
+        print(
+            f"Enter the file path to save the glossary to, or just hit enter for default (glossary.txt):"
+        )
+        path = prompt().strip()
+        if path == "":
+            path = "glossary.txt"
+        with open(path, "w") as f:
+            f.write(glossary)
+        print(f"Your glossary has been saved to {path}.")
+    elif action == "delete":
+        dfg.delete_glossary()
+
+
 def query():
     """
     Run a query and print the results alongside the generated SQL query.
@@ -347,6 +488,15 @@ def query():
     else:
         query = sys.argv[2]
 
+    while len(query) > 300 and query != "e":
+        print(
+            "Your query is too long. If you have general information about your database,"
+            + "you can add it to the glossary to help Defog generate queries more accurately.\n"
+            + "You can add a glossary by running `defog glossary update <path/to/glossary>`.\n"
+            + "You can also use the `defog golden add` command to add golden queries to help Defog generate queries more accurately."
+        )
+        query = prompt("Please enter a shorter query, or type 'e' to exit first:")
+
     user_question = ""
     sql_generated = ""
     while True:
@@ -378,11 +528,15 @@ def query():
             sql_generated = resp.get("query_generated")
             print("Defog generated the following query to answer your question:\n")
             print(f"\033[1m{resp['query_generated']}\033[0m\n")
-            reason_for_query = resp.get("reason_for_query", "")
+
+            reason_for_query = resp.get("reason_for_query")
             if reason_for_query:
                 reason_for_query = reason_for_query.replace(". ", "\n\n")
+
                 print("This was its reasoning for generating this query:\n")
                 print(f"\033[1m{reason_for_query}\033[0m\n")
+            else:
+                reason_for_query = ""
 
             print("Results:\n")
             # print results in tabular format using 'columns' and 'data' keys
@@ -392,10 +546,105 @@ def query():
                 print(resp)
 
             print()
-            get_feedback(df.api_key, df.db_type, user_question, sql_generated)
+            get_feedback(
+                df.api_key, df.db_type, user_question, sql_generated, df.base_url
+            )
         query = prompt("Please enter another query, or type 'e' to exit: ")
 
 
+def golden():
+    """
+    Allow the user to get, add, or delete golden queries.
+    These are used as references during query generation (akin to k-shot learning),
+    and can significantly improve the performance of the query generation model.
+    """
+    # get action from sys.argv or prompt
+    if len(sys.argv) < 3:
+        print(
+            "defog golden requires an action. Please enter 'get', 'add', or 'delete':"
+        )
+        action = prompt().strip().lower()
+    else:
+        action = sys.argv[2].lower()
+    while action not in ["get", "add", "delete", "exit"]:
+        print("Please enter 'get', 'add', 'delete', or 'exit' to exit:")
+        action = prompt().strip().lower()
+
+    dfg = defog.Defog()
+    if action == "get":
+        # get format from sys.argv or prompt
+        if len(sys.argv) < 4:
+            print(
+                "defog golden get requires an export format. Please enter 'json' or 'csv':"
+            )
+            format = prompt().strip().lower()
+        else:
+            format = sys.argv[3].lower()
+        while format not in ["json", "csv"]:
+            print("Please enter 'json' or 'csv':")
+            format = prompt().strip().lower()
+        # make request to get golden queries
+        print("Getting golden queries...")
+        golden_queries = dfg.get_golden_queries(format)
+        if golden_queries:
+            print(f"\nYou have {len(golden_queries)} golden queries:\n")
+        for pair in golden_queries:
+            question = pair["question"]
+            sql = pair["sql"]
+            print(f"Question:\n{question}\nSQL:\n{sql}\n")
+    elif action == "add":
+        # get path from sys.argv or prompt
+        if len(sys.argv) < 4:
+            print(
+                "defog golden add requires a path to a JSON or CSV file containing golden queries:"
+            )
+            path = prompt().strip()
+        else:
+            path = sys.argv[3]
+        while (
+            not os.path.exists(path)
+            and not path.endswith(".json")
+            and not path.endswith(".csv")
+        ):
+            print("File not found. Please enter a valid path:")
+            path = prompt().strip()
+        # if path ends with json, read in json and pass in golden_queries
+        if path.endswith(".json"):
+            with open(path, "r") as f:
+                golden_queries = json.load(f)
+                dfg.update_golden_queries(golden_queries=golden_queries)
+        # if path ends with csv, pass in csv file
+        elif path.endswith(".csv"):
+            dfg.update_golden_queries(golden_queries_path=path)
+    elif action == "delete":
+        # get path from sys.argv or prompt
+        if len(sys.argv) < 4:
+            print(
+                "defog golden delete requires either a path to a JSON or CSV file containing golden queries for deletion, or 'all' to delete all golden queries."
+            )
+            path = prompt().strip()
+        else:
+            path = sys.argv[3]
+        while (
+            not os.path.exists(path)
+            and not path.endswith(".json")
+            and not path.endswith(".csv")
+            and path != "all"
+        ):
+            print(
+                "File not found. Please enter a valid path, or 'all' to delete all golden queries:"
+            )
+            path = prompt().strip()
+        if path == "all":
+            dfg.delete_golden_queries(all=True)
+        elif path.endswith(".json"):
+            with open(path, "r") as f:
+                golden_queries = json.load(f)
+                dfg.delete_golden_queries(golden_queries=golden_queries)
+        elif path.endswith(".csv"):
+            dfg.delete_golden_queries(golden_queries_path=path)
+
+
 def deploy():
     """
     Deploy a cloud function that can be used to run queries.
@@ -493,15 +742,15 @@ def quota():
     """
     df = defog.Defog()
     resp = df.get_quota()
-    if resp["ran_successfully"]:
+    if resp:
         if resp["premium"]:
             print(f"You are currently on the premium plan with unrestricted usage.")
-            print(f"Your current usage is {resp['queries_made']} queries.")
         else:
-            print(
-                f"You are currently on the free plan with {1000-resp['queries_made']} queries remaining for the month."
-            )
-            print(f"Your current usage is {resp['queries_made']} queries.")
+            print(f"You are currently on the free plan with 1000 queries per month.")
+        print("Usage so far:")
+        print(f"{'Month':<10}{'API Calls':<10}")
+        for month, api_calls in resp["data"]:
+            print(f"{month}:  {api_calls}")
     else:
         print(f"Failed to get quota")
 
@@ -561,9 +810,10 @@ def print_table(columns, data):
 
 
 def serve_webserver():
-    from defog.serve import app
     import uvicorn
 
+    from defog.serve import app
+
     uvicorn.run(app, host="localhost", port=1235)
 
 
