@@ -753,13 +753,18 @@ async def _process_openai_response(
     if tools and len(tools) > 0:
         consecutive_exceptions = 0
         while True:
-            total_input_tokens += (
-                response.usage.prompt_tokens
-                - response.usage.prompt_tokens_details.cached_tokens
-            )
-            total_cached_input_tokens += (
-                response.usage.prompt_tokens_details.cached_tokens
-            )
+            # Check if prompt_tokens_details exists in the response
+            if (
+                hasattr(response.usage, "prompt_tokens_details")
+                and response.usage.prompt_tokens_details is not None
+            ):
+                cached_tokens = response.usage.prompt_tokens_details.cached_tokens or 0
+                total_input_tokens += response.usage.prompt_tokens or 0 - cached_tokens
+                total_cached_input_tokens += cached_tokens
+            else:
+                # If prompt_tokens_details doesn't exist, assume all tokens are uncached
+                total_input_tokens += response.usage.prompt_tokens or 0
+
             total_output_tokens += response.usage.completion_tokens
             message = response.choices[0].message
             if message.tool_calls:
@@ -872,10 +877,18 @@ async def _process_openai_response(
             content = response.choices[0].message.content
 
     usage = response.usage
-    total_cached_input_tokens += usage.prompt_tokens_details.cached_tokens
-    total_input_tokens += (
-        usage.prompt_tokens - usage.prompt_tokens_details.cached_tokens
-    )
+    # Check if prompt_tokens_details exists in the response
+    if (
+        hasattr(usage, "prompt_tokens_details")
+        and usage.prompt_tokens_details is not None
+    ):
+        cached_tokens = usage.prompt_tokens_details.cached_tokens or 0
+        total_cached_input_tokens += cached_tokens
+        total_input_tokens += usage.prompt_tokens or 0 - cached_tokens
+    else:
+        # If prompt_tokens_details doesn't exist, assume all tokens are uncached
+        total_input_tokens += usage.prompt_tokens or 0
+
     total_output_tokens += usage.completion_tokens
     return (
         content,
